# -*- coding: utf-8 -*-
from django.db import models


class GeneQuerySet(models.QuerySet):

    def fast(self):
        return self.select_related('chromosome').prefetch_related('synonyms').all()


class TranscriptQuerySet(models.QuerySet):

    def fast(self):
        return self.select_related('gene').all()


class ExonQuerySet(models.QuerySet):

    def fast(self):
        return self.select_related('transcript').all()



# -*- coding: utf-8
from django.contrib import admin

from . import models


class GenomeAdmin(admin.ModelAdmin):
    model = models.Genome
    list_display = ('label', 'active', 'created', 'modified')
    prepopulated_fields = {
        'slug': ('label', )
    }
    search_fields = ('label', 'description')
    list_filter = ('active', )
    save_as = True


class ChromosomeAdmin(admin.ModelAdmin):
    model = models.Chromosome
    list_display = (
        'label', 'genome', 'length',
        'active', 'created', 'modified',
    )
    raw_id_fields = ('genome', )
    search_fields = ('label', )
    list_filter = ('active', )
    save_as = True


class CytoBandAdmin(admin.ModelAdmin):
    model = models.CytoBand
    list_display = (
        'label', 'chromosome', 'start', 'end',
        'active', 'created', 'modified',
    )
    raw_id_fields = ('chromosome', )
    search_fields = ('label', 'stain')
    list_filter = ('active', )
    save_as = True


class GeneAdmin(admin.ModelAdmin):
    model = models.Gene
    list_display = (
        'symbol', 'hgnc_id', 'status', 'chromosome',
        'active', 'created', 'modified',
    )
    raw_id_fields = ('chromosome', 'synonyms')
    search_fields = ('symbol', 'name', 'previous_name', 'synonyms__label')
    list_filter = ('active', )
    save_as = True


class GeneSynonymAdmin(admin.ModelAdmin):
    model = models.GeneSynonym
    list_display = ('label', 'active', 'created', 'modified')
    search_fields = ('label', 'genes__symbol')
    list_filter = ('active', )
    save_as = True


class TranscriptAdmin(admin.ModelAdmin):
    model = models.Transcript
    list_display = (
        'label', 'gene', 'strand',
        'transcription_start', 'transcription_end',
        'cds_start', 'cds_end',
        'active', 'created', 'modified',
    )
    raw_id_fields = ('gene', )
    search_fields = ('label', 'gene__symbol', 'gene__synonyms__label')
    list_filter = ('active', )
    save_as = True


class ExonAdmin(admin.ModelAdmin):
    model = models.Exon
    list_display = (
        'number', 'transcript', 'start', 'end', 'cds',
        'active', 'created', 'modified',
    )
    raw_id_fields = ('transcript', )
    search_fields = (
        'transcript__label',
        'transcript__gene__symbol',
        'transcript__gene__synonyms__label',
    )
    list_filter = ('active', )
    save_as = True


admin.site.register(models.Genome, GenomeAdmin)
admin.site.register(models.Chromosome, ChromosomeAdmin)
admin.site.register(models.CytoBand, CytoBandAdmin)
admin.site.register(models.Gene, GeneAdmin)
admin.site.register(models.GeneSynonym, GeneSynonymAdmin)
admin.site.register(models.Transcript, TranscriptAdmin)
admin.site.register(models.Exon, ExonAdmin)



# -*- coding: utf-8 -*-
from rest_framework import routers

from . import viewsets


router = routers.SimpleRouter()
router.register(r'genes', viewsets.GeneViewSet)
router.register(r'transcripts', viewsets.TranscriptViewSet)
router.register(r'exons', viewsets.ExonViewSet)

default_router = routers.DefaultRouter()
default_router.register(r'genes', viewsets.GeneViewSet)
default_router.register(r'transcripts', viewsets.TranscriptViewSet)
default_router.register(r'exons', viewsets.ExonViewSet)


urlpatterns = default_router.urls



# -*- coding: utf-8 -*-
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import viewsets
from rest_framework.filters import SearchFilter

from . import filters, models, serializers


class GeneViewSet(viewsets.ReadOnlyModelViewSet):
    """ViewSet for viewing Genes."""

    queryset = models.Gene.objects.fast()
    serializer_class = serializers.GeneSerializer
    filter_backends = (DjangoFilterBackend, SearchFilter)
    filter_class = filters.GeneFilter
    search_fields = (
        'symbol',
        'synonyms__label',
    )
    lookup_field = 'symbol__iexact'


class TranscriptViewSet(viewsets.ReadOnlyModelViewSet):
    """ViewSet for viewing Transcripts."""

    queryset = models.Transcript.objects.fast()
    serializer_class = serializers.TranscriptSerializer
    filter_backends = (DjangoFilterBackend, )
    filter_class = filters.TranscriptFilter
    lookup_field = 'label'


class ExonViewSet(viewsets.ReadOnlyModelViewSet):
    """ViewSet for viewing Exons."""

    queryset = models.Exon.objects.fast()
    serializer_class = serializers.ExonSerializer
    filter_backends = (DjangoFilterBackend, )
    filter_class = filters.ExonFilter



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from model_utils import Choices


HGNC_GENE_STATUS = Choices(
    (1, 'approved', _('approved')),
    (2, 'entry_withdrawn', _('entry_withdrawn')),
    (3, 'symbol_withdrawn', _('symbol_withdrawn')),
    (4, 'ucsc_gene', _('ucsc_gene')),
)

STRAND_TYPES = Choices(
    (1, '+', _('+')),
    (2, '-', _('-')),
)



import logging

from django.core.exceptions import ObjectDoesNotExist
from django.core.management import BaseCommand

from genomix.utils import retrieve_compressed_data, retrieve_data

from genome import app_settings, choices, models, utils


logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Sync Genes'

    def add_arguments(self, parser):
        parser.add_argument(
            '--genome-build',
            choices=['hg18', 'hg19', 'hg38'],
            default='hg19',
            help='Genome build to load',
        )

    def handle(self, *args, **options):
        genome = options['genome_build']
        resources = app_settings.RESOURCES.get(genome, {})

        # Download external resources
        logger.info('Downloading resources...')
        hgnc_data = retrieve_data(getattr(app_settings, 'HGNC_GENES'))
        kgXref_data = retrieve_compressed_data(resources.get('kgXref'))

        # Create Genome
        logger.info('Prepping Genome...')
        genome_obj, created = models.Genome.objects.get_or_create(label=genome)

        # Create Gene
        logger.info('Creating Genes from HGNC...')
        hgnc_header = hgnc_data.pop(0).strip().split('\t')

        for line in hgnc_data:
            hgnc_gene_data = line.split('\t')

            symbol = hgnc_gene_data[hgnc_header.index('Approved Symbol')]
            name = hgnc_gene_data[hgnc_header.index('Approved Name')]
            hgnc_id = hgnc_gene_data[hgnc_header.index('HGNC ID')]
            status = hgnc_gene_data[hgnc_header.index('Status')]
            chromosome = hgnc_gene_data[hgnc_header.index('Chromosome')]
            previous_name = hgnc_gene_data[hgnc_header.index('Previous Name')]
            locus_type = hgnc_gene_data[hgnc_header.index('Locus Type')]
            locus_group = hgnc_gene_data[hgnc_header.index('Locus Group')]
            ensembl = hgnc_gene_data[hgnc_header.index('Ensembl Gene ID')]
            refseq = hgnc_gene_data[hgnc_header.index('RefSeq IDs')]
            not_curated_ensembl = hgnc_gene_data[hgnc_header.index('Ensembl ID(supplied by Ensembl)')]
            not_curated_refseq = hgnc_gene_data[hgnc_header.index('RefSeq(supplied by NCBI)')]
            not_curated_ucsc = hgnc_gene_data[hgnc_header.index('UCSC ID(supplied by UCSC)')]
            not_curated_omim = hgnc_gene_data[hgnc_header.index('OMIM ID(supplied by OMIM)')]
            not_curated_uniprot = hgnc_gene_data[hgnc_header.index('UniProt ID(supplied by UniProt)')]
            not_curated_mouse_genome_database = hgnc_gene_data[
                hgnc_header.index('Mouse Genome Database ID(supplied by MGI)')]
            not_curated_rat_genome_database = hgnc_gene_data[
                hgnc_header.index('Rat Genome Database ID(supplied by RGD)')]

            # normalize chromosome
            chromosome = utils.reformat_chromosome(chromosome)
            try:
                chromosome_obj = models.Chromosome.objects.get(
                    genome=genome_obj,
                    label=chromosome,
                )
            except ObjectDoesNotExist:
                chromosome_obj = None
            gene_obj, created = models.Gene.objects.get_or_create(
                symbol=symbol.upper(),
                chromosome=chromosome_obj,
                defaults={
                    'name': name,
                    'hgnc_id': hgnc_id.strip().split(':')[1],
                    'status': getattr(choices.HGNC_GENE_STATUS, status.lower().replace(' ', '_')),
                    'previous_name': previous_name,
                    'locus_type': locus_type,
                    'locus_group': locus_group,
                    'ensembl': ensembl,
                    'refseq': refseq,
                    'not_curated_ensembl': not_curated_ensembl,
                    'not_curated_refseq': not_curated_refseq,
                    'not_curated_ucsc': not_curated_ucsc,
                    'not_curated_omim': not_curated_omim,
                    'not_curated_uniprot': not_curated_uniprot,
                    'not_curated_mouse_genome_database': not_curated_mouse_genome_database,
                    'not_curated_rat_genome_database': not_curated_rat_genome_database,
                }
            )

            # Create GeneSynonym
            synonyms = hgnc_gene_data[
                hgnc_header.index('Synonyms')
            ].strip().split(',')
            previous_symbols = hgnc_gene_data[
                hgnc_header.index('Previous Symbols')
            ].strip().split(',')

            for synonym in synonyms + previous_symbols:
                label = synonym.strip()
                if label:
                    synonym_obj, created = models.GeneSynonym.objects.get_or_create(label=label)
                    gene_obj.synonyms.add(synonym_obj)

            gene_obj.save()

        # Add missing genes from kgXref
        logger.info('Creating Genes from kgXref...')
        for line in kgXref_data:
            line = line.strip().split('\t')
            symbol = line[4]
            refseq = line[5]
            name = line[7]

            gene_obj, created = models.Gene.objects.get_or_create(
                symbol=symbol.upper(),
                defaults={
                    'refseq': refseq,
                    'name': name,
                    'status': getattr(choices.HGNC_GENE_STATUS, 'ucsc_gene')
                }
            )
